<?php
/*
Plugin Name: addImageSizePlugin
Plugin URI: http://www.adjust.ne.jp
Description: 画像サイズ追加プラグイン
Author: ADjust Co.,Ltd.
Author URI: http://www.adjust.ne.jp
Version: 1.0
*/
$addImageSizePlugin = new addImageSizePlugin;

add_filter('plugins_loaded', array($addImageSizePlugin, 'init'));

class addImageSizePlugin
{
  const TABLE_NAME = 'addImageSizePlugin';

  public function init()
  {
    add_filter('activate_'.$this->getPluginBasename(), array($this, 'install'));
    add_filter('deactivate_'.$this->getPluginBasename(), array($this, 'uninstall'));
    add_filter('admin_menu', array($this, 'addMenu'));
    add_filter('intermediate_image_sizes_advanced', array($this, 'hookImageSize'));

    foreach ($this->getSizeAll() as $size)
    {
      add_image_size(sprintf('%sx%s', $size->width, $size->height), $size->width, $size->height, true);
    }
  }

  public function getPluginBasename()
  {
    return plugin_basename(__FILE__);
  }

  public function getTableName()
  {
    global $wpdb;

    return $wpdb->prefix.self::TABLE_NAME;
  }

  public function install()
  {
    $this->createTable();
  }

  public function createTable()
  {
    global $wpdb;
    $tableName = $this->getTableName();
    $wpdb->show_errors();
    if (!$wpdb->get_var(sprintf('SHOW TABLES LIKE "%s"', $tableName)))
    {
      $wpdb->query(sprintf('CREATE TABLE %s (
        id INT(11) NOT NULL AUTO_INCREMENT,
        width INT(11) NOT NULL,
        height INT(11) NOT NULL,
        PRIMARY KEY(id)
      )', $tableName));
      $wpdb->query(sprintf('ALTER TABLE %s ADD UNIQUE (width, height)', $tableName));
    }
    $wpdb->hide_errors();
  }

  public function uninstall()
  {
    $this->dropTable();
  }

  public function dropTable()
  {
    global $wpdb;
    $tableName = $this->getTableName();
    if ($wpdb->get_var(sprintf('SHOW TABLES LIKE "%s"', $tableName)))
    {
      $wpdb->query(sprintf('DROP TABLE %s', $tableName));
    }
  }

  public function addMenu()
  {
    add_options_page('addImageSizePlugin Config', '画像サイズ設定', 'manage_options', $this->getPluginBasename(), function(){
      require 'settingTemplate.php';
    });
  }

  public function setSize($list)
  {
    $this->setEmpty();

    if (is_array($list))
    {
      foreach ($list as $size)
      {
        $this->save($size);
      }
    }
  }

  public function save($size)
  {
    global $wpdb;
    $tableName = $this->getTableName();

    if ($this->isValidSize($size))
    {
      $result = $wpdb->query($wpdb->prepare("INSERT INTO `$tableName` VALUES (NULL, %s, %s)", array($size['width'], $size['height'])));
    }
  }

  public function isValidSize($size)
  {
    if (empty($size['width']) || empty($size['height']))
    {
      return false;
    }

    global $wpdb;
    $tableName = $this->getTableName();
    $row = $wpdb->get_row(
      $wpdb->prepare("SELECT * FROM `$tableName` WHERE width = %s AND height = %s", array($size['width'], $size['height']))
    );

    return empty($row);
  }

  public function getSizeAll()
  {
    global $wpdb;
    $tableName = $this->getTableName();

    return $wpdb->get_results(
      $wpdb->prepare("SELECT * FROM `$tableName`", null)
    );
  }

  public function setEmpty()
  {
    global $wpdb;
    $tableName = $this->getTableName();
    $row = $wpdb->get_row(
      $wpdb->prepare("TRUNCATE `$tableName`", null)
    );
  }

  public function hookImageSize($sizes)
  {
    unset($sizes['thumbnail']);
    unset($sizes['medium']);
    unset($sizes['large']);

    return $sizes;
  }
}
